"""
test_data.py

Testing suite for solary/neo/data.py

"""

# Import standard libraries
import sqlite3

# Import installed libraries
import pytest

# Import solary
import solary


def test__get_neodys_neo_nr():
    """
    Testing the hidden function that gets the current number of known NEOs from the NEODyS webpage.

    Returns
    -------
    None.

    """

    # Call the function to get the number of currently known NEOs.
    neo_nr = solary.neo.data._get_neodys_neo_nr()

    # Check of the result is an integer
    assert isinstance(neo_nr, int)

    # Since the number of NEOs can change daily this assertion test check only if the number is
    # larger than 0
    assert neo_nr >= 0


def test_download():
    """
    Testing the NEODyS download function

    Returns
    -------
    None.

    """

    # Execute the download function and check if the download status was "OK"
    dl_status, _ = solary.neo.data.download()
    assert dl_status == 'OK'

    # Execute the donwload a second time get also the row expectation (internally it compares the
    # results with the _get_neodys_neo_nr function)
    dl_status, row_exp = solary.neo.data.download(row_exp=True)
    assert dl_status == 'OK'
    assert isinstance(row_exp, int)
    assert row_exp >= 0


def test_read_neodys():
    """
    Test the reading functionality of the downloaded NEODyS data

    Returns
    -------
    None.

    """

    # Read the data
    neo_dict_data = solary.neo.data.read_neodys()

    # The first entry must be (433) Erors; with its semi-major axis givne in AU and the
    # eccentricity
    assert neo_dict_data[0]['Name'] == '433'
    assert pytest.approx(neo_dict_data[0]['SemMajAxis_AU'], abs=1e-2) == 1.46
    assert pytest.approx(neo_dict_data[0]['Ecc_'], abs=1e-2) == 0.22


def test_NEOdysDatabase():
    """
    Test the NEODyS database.

    Returns
    -------
    None.

    """

    # Create the database and check if the connection is established and the cursor is set
    neo_sqlite = solary.neo.data.NEOdysDatabase(new=True)

    assert isinstance(neo_sqlite.con, sqlite3.Connection)
    assert isinstance(neo_sqlite.cur, sqlite3.Cursor)

    # Create the table and content
    neo_sqlite.create()

    # Get now the data of (433) Eros
    query_res_cur = neo_sqlite.cur.execute('SELECT Name, SemMajAxis_AU, ECC_ ' \
                                           'FROM main WHERE Name = "433"')
    query_res = query_res_cur.fetchone()

    # Perform assertion tests on Eros' data
    assert query_res[0] == '433'
    assert pytest.approx(query_res[1], abs=1e-2) == 1.46
    assert pytest.approx(query_res[2], abs=1e-2) == 0.22

    # Compute the derived orbital elements and the the aphelion and perihelion data of Eros
    neo_sqlite.create_deriv_orb()
    query_res_cur = neo_sqlite.cur.execute('SELECT Name, Aphel_AU, Perihel_AU ' \
                                           'FROM main WHERE Name = "433"')
    query_res = query_res_cur.fetchone()

    # Perform assertion tests on Eros' derived results
    assert query_res[0] == '433'
    assert pytest.approx(query_res[1], abs=1e-3) == 1.783
    assert pytest.approx(query_res[2], abs=1e-3) == 1.133

    # Now the test check if the update functionality works. For this purpose, the first row from the
    # database is deleted; the update function is executed and then the number of rows is compared
    # with the expectation.
    # First: get the current number of rows from the database
    query_res_cur = neo_sqlite.cur.execute('SELECT COUNT(*) FROM main')
    original_count = query_res_cur.fetchone()[0]

    # Delete Eros and get the number of rows
    neo_sqlite.cur.execute('DELETE FROM main WHERE Name="433"')
    neo_sqlite.con.commit()
    query_res_cur = neo_sqlite.cur.execute('SELECT COUNT(*) FROM main')
    manip_count = query_res_cur.fetchone()[0]

    # The modified database should have 1 entry less than before
    assert manip_count == original_count-1

    # Update the database and get the number of counts
    neo_sqlite.update()
    query_res_cur = neo_sqlite.cur.execute('SELECT COUNT(*) FROM main')
    update_count = query_res_cur.fetchone()[0]

    # Now the updated database must have the same number of rows as before
    assert update_count == original_count

    # Close the database
    neo_sqlite.close()


def test_download_granvik2018():
    """
    Testing the download of the Granvik et al. (2018) NEO data.

    Returns
    -------
    None.

    """

    # Download the data and compare the MD5 hash of the downloaded file with the expectation
    md5_hash = solary.neo.data.download_granvik2018()
    assert md5_hash == '521ddfdc18545c736fee36dbc4879d5e'


def test_read_granvik2018():
    """
    Test the reader function of the Granvik et al. (2018) data.

    Returns
    -------
    None.

    """

    # Read the data and perform some assertions (expectations for the very first entry that has
    # been inspected before)
    neo_dict_data = solary.neo.data.read_granvik2018()
    assert pytest.approx(neo_dict_data[0]['SemMajAxis_AU']) == 2.57498121
    assert pytest.approx(neo_dict_data[0]['Ecc_']) == 0.783616960
    assert pytest.approx(neo_dict_data[0]['Incl_deg']) == 33.5207634
    assert pytest.approx(neo_dict_data[0]['LongAscNode_deg']) == 278.480591
    assert pytest.approx(neo_dict_data[0]['ArgP_deg']) == 75.9520569
    assert pytest.approx(neo_dict_data[0]['MeanAnom_deg']) == 103.833748
    assert pytest.approx(neo_dict_data[0]['AbsMag_']) == 21.0643673


def test_Granvik2018Database():
    """
    Test the Granvik et al. (2018) SQLite database

    Returns
    -------
    None.

    """

    # Create the database
    granvik2018_sqlite = solary.neo.data.Granvik2018Database(new=True)

    # Check if a connection has been established and if a cursor has been set
    assert isinstance(granvik2018_sqlite.con, sqlite3.Connection)
    assert isinstance(granvik2018_sqlite.cur, sqlite3.Cursor)

    # Create the main table, get the first entry and verify the results
    granvik2018_sqlite.create()
    query_res_cur = granvik2018_sqlite.cur.execute('SELECT ID, SemMajAxis_AU, ECC_ ' \
                                                    'FROM main WHERE ID = 1')
    query_res = query_res_cur.fetchone()
    assert query_res[1] == 2.57498121
    assert query_res[2] == 0.783616960

    # Create the dervied orbital elements and perform a verfication step
    granvik2018_sqlite.create_deriv_orb()
    query_res_cur = granvik2018_sqlite.cur.execute('SELECT ID, Aphel_AU, Perihel_AU ' \
                                                    'FROM main WHERE ID = 1')
    query_res = query_res_cur.fetchone()
    assert query_res[1] == 4.592780157837321
    assert query_res[2] == 0.5571822621626783

    # Close the Granvik database
    granvik2018_sqlite.close()



"""
test_photometry.py

Testing suite for solary/general/photometry.py

"""

# Import standard libraries
import math

# Import installed libraries
import pytest

# Import solary
import solary


def test_appmag2irr():
    """
    Testing the function appmag2irr that converts the apparent magnitude to an irradiance in W/m^2

    Returns
    -------
    None.

    """

    # Read the constants file and get the bolometric zero value
    config = solary.auxiliary.config.get_constants()
    appmag_irr_i0 = float(config['photometry']['appmag_irr_i0'])

    # The apparent magnitude of 0 mag must correspond to the bolometric 0 value
    irradiance1 = solary.general.photometry.appmag2irr(app_mag=0)
    assert pytest.approx(irradiance1) == appmag_irr_i0

    # -5 mag must be 100 brighter than the bolometric zero value
    irradiance2 = solary.general.photometry.appmag2irr(app_mag=-5)
    assert pytest.approx(irradiance2) == appmag_irr_i0 * 100.0

    # Another example
    irradiance3 = solary.general.photometry.appmag2irr(app_mag=1)
    assert pytest.approx(irradiance3) == 1.0024422165005002e-08


def test_intmag2surmag():
    """
    Testing the function intmag2surmag that converts the integrated magnitude and over all area of
    the object to a surface brightness given in mag/arcsec^2.

    Returns
    -------
    None.

    """

    # Set a simple example, since the equation is quite trivial
    surf_bright = solary.general.photometry.intmag2surmag(intmag=10.0, area=10.0)
    assert surf_bright == 12.5


def test_surmag2intmag():
    """
    Testing the function surmag2intmag that converts the surface brightness and area to a
    corresponding integrated magnitude.

    Returns
    -------
    None.

    """

    # Set the same example as shown for the function intmag2surmag
    intmag = solary.general.photometry.surmag2intmag(surmag=12.5, area=10.0)
    assert intmag == 10.0


def test_phase_func():
    """
    This function tests the phase function that is needed for the apparent magnitude computation of
    minor bodies

    Returns
    -------
    None.

    """

    # Example 1
    phi1_res1 = solary.general.photometry.phase_func(index=1, phase_angle=0.0)
    assert phi1_res1 == 1.0

    # Example 2
    phi2_res1 = solary.general.photometry.phase_func(index=1, phase_angle=0.0)
    assert phi2_res1 == 1.0

    # Example 3
    phi1_res2 = solary.general.photometry.phase_func(index=1, phase_angle=math.pi/2.0)
    assert phi1_res2 == 0.03579310506765532

    # Example 4
    phi2_res2 = solary.general.photometry.phase_func(index=2, phase_angle=math.pi/2.0)
    assert phi2_res2 == 0.15412366181513143


def test_reduc_mag():
    """
    Testing the function that computed the reduced magnitude.

    Returns
    -------
    None.

    """

    # An absolute magnitude of 0 and a phase angle of 0 degrees must correspond to a reduced
    # magnitude of 0
    red_mag1 = solary.general.photometry.reduc_mag(abs_mag=0, slope_g=0.15, phase_angle=0.0)
    assert red_mag1 == 0.0

    # Second artifically set and computed example
    red_mag2 = solary.general.photometry.reduc_mag(abs_mag=0, slope_g=0.15, phase_angle=math.pi/2.0)
    assert red_mag2 == 3.178249562605391


def test_hg_app_mag():
    """
    Testing the computation of the apparent magnitude (H-G system).

    Returns
    -------
    None.

    """

    # Set sample vectors (example: first vector is an asteroid at 2 AU, second vector is planet
    # Earth at 1 AU)
    vec_obj1 = [2.0, 0.0, 0.0]
    vec_obs1 = [1.0, 0.0, 0.0]

    # Compute the vector object -> observer
    vec_obj2obs1 = solary.general.vec.substract(vector1=vec_obs1, vector2=vec_obj1)

    # Compute the vector object -> illumination source (by inversing the object vector to
    # [-2.0, 0.0, 0.0])
    vec_obj2ill1 = solary.general.vec.inverse(vector=vec_obj1)

    # Compute the apparent magnitude for an object with an absolute magnitude of 0.0 mag and a
    # slope parameter of 0.15 (default value for asteroids).
    app_mag1 = solary.general.photometry.hg_app_mag(abs_mag=0.0, \
                                                    slope_g=0.15, \
                                                    vec_obj2obs=vec_obj2obs1, \
                                                    vec_obj2ill=vec_obj2ill1)
    assert app_mag1 == 1.505149978319906


    # Second example with simplified values for (1) Ceres (at opposition w.r.t. Earth)
    vec_obj2 = [3.0, 0.0, 0.0]
    vec_obs2 = [1.0, 0.0, 0.0]

    vec_obj2obs2 = solary.general.vec.substract(vector1=vec_obs2, \
                                                vector2=vec_obj2)
    vec_obj2ill2 = solary.general.vec.inverse(vector=vec_obj2)

    app_mag2 = solary.general.photometry.hg_app_mag(abs_mag=3.4, \
                                                    slope_g=0.12, \
                                                    vec_obj2obs=vec_obj2obs2, \
                                                    vec_obj2ill=vec_obj2ill2)

    # Ceres has a brightness of around 7 mag at opposition
    assert app_mag2 == 7.290756251918218

    # Third example with the same values but a larger phase angle
    vec_obj3 = [0.0, 3.0, 0.0]
    vec_obs3 = [1.0, 0.0, 0.0]

    vec_obj2obs3 = solary.general.vec.substract(vector1=vec_obs3, \
                                                vector2=vec_obj3)
    vec_obj2ill3 = solary.general.vec.inverse(vector=vec_obj3)

    app_mag3 = solary.general.photometry.hg_app_mag(abs_mag=3.4, \
                                                    slope_g=0.12, \
                                                    vec_obj2obs=vec_obj2obs3, \
                                                    vec_obj2ill=vec_obj2ill3)

    # A larger phase angle should lead to a smaller brightness (larger apparent magnitude);
    # compared to the opposition result
    assert app_mag3 > app_mag2



"""
test_config.py

Testing suite for solary/auxiliary/config.py

"""

# Import solary
import solary


def test_get_constants():
    """
    Test function to check whether the constants file is read successfully

    Returns
    -------
    None.

    """

    # Call the constants get function
    constant_config = solary.auxiliary.config.get_constants()

    # If the reading was successful the config object shall have miscellaneous sections and
    # corresponding values. One of them is called "constants"
    constant_config_sections = constant_config.sections()
    assert 'constants' in constant_config_sections


def test_get_paths():
    """
    Test function to check whether the path config file is read.

    Returns
    -------
    None.

    """

    # Call the paths config file
    paths_config = solary.auxiliary.config.get_paths()

    # If the reading was successful the config object shall have miscellaneous sections and
    # corresponding values. One of them is called "neo"
    paths_config_sections = paths_config.sections()
    assert 'neo' in paths_config_sections

    # Testing now the test config
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    # If the reading was successful the config object shall have miscellaneous sections and
    # corresponding values. One of them contains "instruments_telescope_optical"
    test_paths_config_sections = test_paths_config.sections()
    assert 'instruments_optics_reflector' in test_paths_config_sections



# pylint: disable=no-member
"""
test_optics.py

Testing suite for solary/instruments/optics.py

"""

# Import installed libraries
import pytest

# Import solary
import solary


# Define a test fixture that is being used in all tests. The fixture loads configuration files.
@pytest.fixture(name='reflector_test_optics')
def fixture_reflector_test_optics():
    """
    Fixture to load the test configuration files.

    Returns
    -------
    test_reflector_dict : dict
        Reflector test configuration.

    """

    # Get the test config file paths
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    # Get the path to the relfector config file
    test_reflector_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['instruments_optics_reflector']['properties'])

    # Read and parse the reflector config file and return a dictionary with the properties
    test_reflector_dict = solary.instruments.optics.read_reflector_config(test_reflector_path)

    return test_reflector_dict


def test_read_optical_config(reflector_test_optics):
    """
    Testing if the config file reading was successful.

    Parameters
    ----------
    reflector_test_optics : dict
        Reflector config dictionary.

    Returns
    -------
    None.

    """

    # Check if the fixture load is a dictionary
    assert isinstance(reflector_test_optics, dict)

    # Check the main mirror entry
    assert reflector_test_optics['main_mirror_dia'] == 1.0


def test_reflector(reflector_test_optics):
    """
    Testing the Reflector Class.

    Parameters
    ----------
    reflector_test_optics : dict
        Reflector config dictionary.

    Returns
    -------
    None.

    """

    # Initiate the Reflector class
    test_reflector_class = solary.instruments.optics.Reflector(reflector_test_optics)

    # Check if the instances of the class correspond to the config file
    assert test_reflector_class.main_mirror_dia == reflector_test_optics['main_mirror_dia']
    assert test_reflector_class.sec_mirror_dia == reflector_test_optics['sec_mirror_dia']
    assert test_reflector_class.optical_throughput == reflector_test_optics['optical_throughput']
    assert test_reflector_class.focal_length == reflector_test_optics['focal_length']

    # Check the main mirror area that depends on the diameter of the main mirror
    assert test_reflector_class.main_mirror_area == \
        solary.general.geometry.circle_area(radius=reflector_test_optics['main_mirror_dia'] / 2.0)

    # Check the secondary mirror area that depends on the diameter of the secondary mirror
    assert test_reflector_class.sec_mirror_area == \
        solary.general.geometry.circle_area(radius=reflector_test_optics['sec_mirror_dia'] / 2.0)

    # Check now the total collect area
    assert test_reflector_class.collect_area == \
        test_reflector_class.main_mirror_area - test_reflector_class.sec_mirror_area



"""
vec.py

Auxiliary functions for vector computations.

"""

# Import standard libraries
import math


def norm(vector):
    """
    This function computes the norm of a given vector. The current version computes only the
    Euclidean Norm, respectivels the p2 norm.

    Parameters
    ----------
    vector : list
        Input vector of any dimensionality.

    Returns
    -------
    norm_res : float
        Norm of the input vector.

    Examples
    --------
    >>> import solary
    >>> vec_norm = solary.general.vec.norm(vector=[3.0, 5.0, -5.9])
    >>> vec_norm
    8.295179322956196

    """

    # Compute the norm by summing all squared elements
    norm_res = math.sqrt(sum(abs(elem)**2.0 for elem in vector))

    return norm_res


def unify(vector):
    """
    This function normalises the input vector. So, the elements of the vector are divided by the
    norm of the vector. The result is a unit vector with the length 1.

    Parameters
    ----------
    vector : list
        Input vector of any dimensionality.

    Returns
    -------
    unit_vector : list
        Unified / Normalised vector.

    Examples
    --------
    >>> import solary
    >>> unit_vec = solary.general.vec.unify(vector=[1.0, 5.0, 10.0])
    >>> unit_vec
    [0.0890870806374748, 0.44543540318737396, 0.8908708063747479]

    Now check the norm of the resulting vector

    >>> vec_norm = solary.general.vec.norm(vector=unit_vec)
    >>> vec_norm
    1.0

    """

    # Compute the norm of the input vector
    vector_norm = norm(vector)

    # Iterate through all input vector elements and normalise them
    unit_vector = [vector_elem / vector_norm for vector_elem in vector]

    return unit_vector


def dot_prod(vector1, vector2):
    """
    This function computes the dot product between two given vectors.

    Parameters
    ----------
    vector1 : list
        Input vector #1 of any dimensionality.
    vector2 : list
        Input vector #2 with the same dimensionality as vector1.

    Returns
    -------
    dotp_res : float
        Dot product of both vectors.

    Examples
    --------
    >>> import solary
    >>> dot_product_res = solary.general.vec.dot_prod(vector1=[1.5, -4.0, 8.0], \
                                                      vector2=[-5.0, -4.20, 0.0])

    >>> dot_product_res
    9.3

    """

    # Compute dot product
    dotp_res = sum(v1_i * v2_i for v1_i, v2_i in zip(vector1, vector2))

    return dotp_res


def phase_angle(vector1, vector2):
    """
    This function compute the phase angle between two vectors. The phase angle is the enclosed
    angle between the vectors at their corresponding point of origin.

    The output is given in radians and ranges from 0 to pi.

    Parameters
    ----------
    vector1 : list
        Input vector #1 of any dimensionality.
    vector2 : list
        Input vector #2 with the same dimensionality as vector1.

    Returns
    -------
    angle_rad : float
        Phase angle in radians between vector1 and vector2.

    Examples
    --------
    >>> import math
    >>> import solary
    >>> ph_angle_rad = solary.general.vec.phase_angle(vector1=[1.0, 0.0], \
                                                      vector2=[1.0, 1.0])
    >>> ph_angle_deg = math.degrees(ph_angle_rad)
    >>> ph_angle_deg
    45.0

    """

    # Compute the phase angle by considering the rearranged, known geometric definition of the dot
    # product
    angle_rad = math.acos(dot_prod(vector1, vector2) \
                          / (norm(vector1) * norm(vector2)))

    return angle_rad


def substract(vector1, vector2):
    """
    This function substracts the vector elements of one list with the elements of another list.
    Alternatively, one can use the Numpy library and Numpy arrays without using this function at
    all.

    Parameters
    ----------
    vector1 : list
        Input vector #1 of any dimensionality.
    vector2 : list
        Input vector #2 with the same dimensionality as vector1.

    Returns
    -------
    diff_vector : list
        Difference vector with the same dimensionality as vector1.

    Examples
    --------
    >>> import solary
    >>> vector_diff = solary.general.vec.substract(vector1=[1.0, 4.0, 2.0], \
                                                   vector2=[-8.0, 0.0, 1.0])
    >>> vector_diff
    [9.0, 4.0, 1.0]

    """

    # Set an empty list for the vector difference / substraction
    diff_vector = []

    # Zip both input vector
    zipped_vector = zip(vector1, vector2)

    # Iterate through all elements and compute the substraction
    for vector1_i, vector2_i in zipped_vector:
        diff_vector.append(vector1_i - vector2_i)

    return diff_vector


def inverse(vector):
    """
    Inverse the vector's elements. Alternatively, apply -1 on a Numpy array.

    Parameters
    ----------
    vector : list
        Input vector of any dimensionality.

    Returns
    -------
    inv_vector : TYPE
        Inverse output vector with the same dimensionality as vector.

    Examples
    --------
    >>> import solary
    >>> inverse_vector = solary.general.vec.inverse(vector=[1.0, 2.0, -3.0])
    >>> inverse_vector
    [-1.0, -2.0, 3.0]

    """

    # Inverse the vector element entries by multiplying -1.0 to each element
    inv_vector = [-1.0 * vector_elem for vector_elem in vector]

    return inv_vector



"""
telescope.py

Script that contains telescope classes that mostly consists of base classes (e.g., optical systems
                                                                             cameras, etc.)

"""

# Import standard libraries
import math

# Import solary
import solary

# Import the camera and optics script from the same directory
from . import camera
from . import optics


def comp_fov(sensor_dim, focal_length):
    """
    Compute the Field-Of-View (FOV) depending on a camera's chip size and telescope's focal length.
    See [1].

    Parameters
    ----------
    sensor_dim : float
        Sensor size (e.g., x or y dimension). Given in mm.
    focal_length : float
        Focal length of the telescope. Given in mm.

    Returns
    -------
    fov_arcsec : float
        Resulting FOV. Given in arcsec.

    References
    ----------
    [1] https://www.celestron.com/blogs/knowledgebase/
        how-do-i-determine-the-field-of-view-for-my-ccd-chip-and-telescope; 04.Jan.2021

    """

    # Compute the FOV. The equation, provided by [1], has been converted from imperial to SI.
    fov_arcsec = (3436.62 * sensor_dim / focal_length) * 60.0

    return fov_arcsec


class ReflectorCCD(optics.Reflector, camera.CCD):
    """
    Class of a reflector telescope with a CCD camera system. This class loads config files and sets
    attributes of a telescope system. Further, one can set observationsl settings like e.g., the
    exposure time. Functions allow one to compute the Signal-To-Noise ratio of an object.

    Since this class is build on the Reflector and CCD class, please check "See Also" to see the
    class references. These base classes contain more attributes, properties, etc. and can be read
    in the corresponding docstring.

    Attributes
    ----------
    _photo_flux_v : float
        Photon flux of a 0 mag star in V-Band.

    Static Properties
    -----------------
    fov : list
        FOV values (x and y dimension of the CCD chip). Given in arcsec.
    ifov_res : list
        iFOV values (x and y dimension of the CCD chip). Given in arcsec / pixel.

    Dynamic Properties
    ------------------
    aperture : float
        Aperture for photometric / astrometric purposes. Given in arcsec.
    hfdia : float
        Half Flux Diameter. Given in arcsec.
    exposure_time : float
        Exposure time. Given in s.
    pixels_in_aperture : int
        Number of pixels within the aperture (rounded).
    dark_esignal_aperture : float
        Number of dark current electrons.

    Methods
    -------
    object_esignal(mag)
        Compute the number of CCD electrons that are created by the object's brightness within the
        photometric aperture.
    sky_esignal(mag_arcsec_sq)
        Compute the number of CCD electrons that are created by the background sky brighness within
        the photometric aperture (considering only the included pixels).
    object_snr(obj_mag, sky_mag_arcsec_sq)
        Compute the Signal-To-Noise ratio of an object.

    See Also
    --------
    solary.instruments.optics.Reflector
    solary.instruments.camera.CCD

    """


    def __init__(self, optics_config, ccd_config):
        """
        Init function.

        Parameters
        ----------
        optics_config : dict
            Reflector config.
        ccd_config : dict
            CCD config.

        Returns
        -------
        None.

        See also
        --------
        solary.instruments.optics.Reflector :
            The Reflector base class that contains the optics attributes and properties.
        solary.instruments.camera.CCD :
            The CCD base class that contains the camera attributes and properties.

        """

        # Init the optics and camera classes accordingly
        optics.Reflector.__init__(self, optics_config)
        camera.CCD.__init__(self, ccd_config)

        # Load the constants config file and get the photon flux (Given in m^-2 * s^-1)
        config = solary.auxiliary.config.get_constants()
        self._photon_flux_v = float(config['photometry']['photon_flux_V'])


    @property
    def fov(self):
        """
        Get the Field-Of-View (FOV) of the telescope in x and y dimensions.

        Returns
        -------
        fov_res : list
            FOV values (x and y dimension of the CCD chip). Given in arcsec.

        """

        # Placholder list for the results
        fov_res = []

        # Iterate through the chip size list (given in mm) and multiply the focal length that is
        # given in meters with 1000 to convert it to mm.
        for chip_dim in self.chip_size:
            fov_res.append(comp_fov(sensor_dim=chip_dim, focal_length=self.focal_length*1000.0))

        return fov_res


    @property
    def ifov(self):
        """
        Get the individual Field-Of-View (iFOV). The iFOV is the FOV that applies for each pixel.

        Returns
        -------
        ifov_res : list
            iFOV values (x and y dimension of the CCD chip). Given in arcsec / pixel.

        """

        # Placeholder list for the iFOV results
        ifov_res = []

        # Iterate through the FOV values and number of pixels. Divide the FOV dimension by the
        # corresponding number of pixels
        for fov_dim, pixel_dim in zip(self.fov, self.pixels):
            ifov_res.append(fov_dim / pixel_dim)

        return ifov_res


    @property
    def aperture(self):
        """
        Get the aperture.

        Returns
        -------
        float
            Photometry aperture. Given in arcsec.

        """

        return self._aperture


    @aperture.setter
    def aperture(self, apert):
        """
        Set the aperture

        Parameters
        ----------
        apert : float
            Aperture for photometric / astrometric purposes. Given in arcsec.

        Returns
        -------
        None.

        """

        self._aperture = apert


    @property
    def hfdia(self):
        """
        Get the half flux diameter of an object / star.

        Returns
        -------
        float
            Half Flux Diameter. Given in arcsec.

        """

        return self._hfdia


    @hfdia.setter
    def hfdia(self, halfflux_dia):
        """
        Set the half flux diameter of an object / star.

        Parameters
        ----------
        halfflux_dia : float
            Half Flux Diameter. Given in arcsec.

        Returns
        -------
        None.

        """

        self._hfdia = halfflux_dia


    @property
    def exposure_time(self):
        """
        Get the exposure time.

        Returns
        -------
        float
            Exposure time. Given in s.

        """

        return self._exposure_time


    @exposure_time.setter
    def exposure_time(self, exp_time):
        """
        Set the exposure time.

        Parameters
        ----------
        exp_time : float
            Exposure time. Given in s.

        Returns
        -------
        None.

        """

        self._exposure_time = exp_time


    @property
    def pixels_in_aperture(self):
        """
        Get the number of pixels within the photometric aperture.

        Returns
        -------
        pixels_in_aperture : int
            Number of pixels within the aperture (rounded).

        """

        # Number of pixels corresponds to the aperture area (assuming a cirlce) divided by the iFOV
        frac_pixels_in_aperture = solary.general.geometry.circle_area(0.5 * self.aperture) \
                                  / math.prod(self.ifov)

        # Round the result
        pixels_in_aperture = int(round(frac_pixels_in_aperture, 0))

        return pixels_in_aperture


    @property
    def _ratio_light_aperture(self):
        """
        Get the ratio of light that is collected within the photometric aperture.
        Assumption: the light of an object is Gaussian distributed (in both directions equally);
                    the ration within the photometric aperture can be perfectly described by the
                    error functon.

        Returns
        -------
        _ratio : float
            Ratio of light within the photometric aperture. Value between 0 and 1.

        """

        # Compute the Gaussian standard deviation of the half flux diameter in arcsec
        sigma = solary.general.geometry.fwhm2std(self.hfdia)

        # Compute the ratio, using the error function, the photometric aperture and half flux
        # diameter corresponding standard deviation
        _ratio = math.erf(self.aperture / (sigma * math.sqrt(2))) ** 2.0

        return _ratio


    def object_esignal(self, mag):
        """
        Function to compute the number of the object's corresponding electrons that are created
        within the photometric aperture on the CCD.

        Parameters
        ----------
        mag : float
            Brightness of the object in V-Band. Given in mag.

        Returns
        -------
        obj_sig_aper : float
            Number of electrons that are created within the aperture.

        """

        # Compute the number of electrons:
        #    1. Scale by the magnitude
        #    2. Multiply by the photon flux in V-band
        #    3. Multiply by the exposure time in seconds
        #    4. Multiply by the telescope's collection area
        #    5. Multiply by the light ratio within the aperture
        #    6. Multiply by the quantum efficiency
        #    7. Multiply by the optical throughput
        obj_sig_aper = 10.0 ** (-0.4 * mag) \
                       * self._photon_flux_v \
                       * self.exposure_time \
                       * self.collect_area \
                       * self._ratio_light_aperture \
                       * self.quantum_eff \
                       * self.optical_throughput

        # Round the result
        obj_sig_aper = round(obj_sig_aper, 0)

        return obj_sig_aper


    def sky_esignal(self, mag_arcsec_sq):
        """
        Function to compute the number of electrons within the photometric aperture caused by the
        background sky brightness.

        Parameters
        ----------
        mag_arcsec_sq : float
            Background sky brightness in V-Band. Given in mag/arcsec^2

        Returns
        -------
        sky_sig_aper : float
            Number of electrons that are created within the aperture.

        """

        # First, convert the sky surface brightness to an integrated brightness (apply the complete
        # telescope's collection area)
        total_sky_mag = solary.general.photometry.surmag2intmag(surmag=mag_arcsec_sq, \
                                                                area=math.prod(self.fov))

        # Compute the number of electrons:
        #    1. Scale by the magnitude
        #    2. Multiply by the photon flux in V-band
        #    3. Multiply by the exposure time in seconds
        #    4. Multiply by the telescope's collection area
        #    5. Multiply by the number of pixels within the aperture w.r.t. the total number of
        #       pixels (discrete ratio)
        #    6. Multiply by the quantum efficiency
        #    7. Multiply by the optical throughput
        sky_sig_aper = 10.0 ** (-0.4 * total_sky_mag) \
                       * self._photon_flux_v \
                       * self.exposure_time \
                       * self.collect_area \
                       * (self.pixels_in_aperture / math.prod(self.pixels)) \
                       * self.quantum_eff \
                       * self.optical_throughput

        # Round the result
        sky_sig_aper = round(sky_sig_aper, 0)

        return sky_sig_aper


    @property
    def dark_esignal_aperture(self):
        """
        Get the number of dark current induced electrons within the aperture

        Returns
        -------
        dark_sig_aper : float
            Number of dark current electrons.

        """

        # Compute the number of dark current electrons (Noise * exposure time * number of pixels
        # within the photometric aperture)
        dark_sig_aper = self.dark_noise * self.exposure_time * self.pixels_in_aperture

        # Round the result
        dark_sig_aper = round(dark_sig_aper, 0)

        return dark_sig_aper


    def object_snr(self, obj_mag, sky_mag_arcsec_sq):
        """
        Compute the Signal-To-Noise ratio (SNR) of an object.

        Parameters
        ----------
        obj_mag : float
            Object brightness. Given in mag.
        sky_mag_arcsec_sq : float
            Background sky brightness. Given in mag/arcsec^2.

        Returns
        -------
        snr : float
            SNR of the object.

        """

        # Compute the signal of the object (electrons within the photometric aperture)
        signal = self.object_esignal(mag=obj_mag)

        # Compute the noise
        noise = math.sqrt(signal
                          + self.sky_esignal(mag_arcsec_sq=sky_mag_arcsec_sq)
                          + self.dark_esignal_aperture)

        # Determine the SNR
        snr = signal/noise

        return snr



"""
test_astrodyn.py

Testing suite for solary/general/astrodyn.py

"""

# Import standard libraries
import math

# Import installed libraries
import pytest

# Import solary
import solary


@pytest.fixture(name='test_orbit_data')
def fixture_test_orbit_data():
    """
    Fixture to load orbit example data.

    Returns
    -------
    test_orbit_values : dict
        Dictionary that contains the values.
    test_orbit_units : dict
        Dictionary that contains the units.

    """

    # Get the test config file paths
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    test_orbit_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['general_astrodyn']['base_class_orbit'])

    test_orbit_values, test_orbit_units = \
        solary.auxiliary.reader.read_orbit(test_orbit_path)

    return test_orbit_values, test_orbit_units


def test_tisserand():
    """
    Test function for the Tisserand computation function.

    Returns
    -------
    None.

    """

    # Compute Tisserand parameter (test case #1) and compare with expectation
    tisserand_parameter1 = solary.general.astrodyn.tisserand(sem_maj_axis_obj=5.0, \
                                                             inc=0.0, \
                                                             ecc=0.0)
    assert tisserand_parameter1 == 3.001200087241328

    # Compute Tisserand parameter (test case #2) and compare with expectation
    tisserand_parameter2 = solary.general.astrodyn.tisserand(sem_maj_axis_obj=4.0, \
                                                             inc=0.0, \
                                                             ecc=0.65)
    assert tisserand_parameter2 == 2.633422691976387

    # Compute Tisserand parameter (test case #3) and compare with expectation
    tisserand_parameter3 = solary.general.astrodyn.tisserand(sem_maj_axis_obj=4.0, \
                                                             inc=math.radians(30.0), \
                                                             ecc=0.65)
    assert tisserand_parameter3 == 2.454890564710888

    # Compute Tisserand parameter (test case #4) and compare with expectation
    tisserand_parameter4 = solary.general.astrodyn.tisserand(sem_maj_axis_obj=4.0, \
                                                             inc=math.radians(30.0), \
                                                             ecc=0.65, \
                                                             sem_maj_axis_planet=3.0)
    assert tisserand_parameter4 == 2.2698684153570663


def test_kep_apoapsis():
    """
    Test function for the Apoapsis computation.

    Returns
    -------
    None.

    """

    # Compute the Apoapsis and perform assertion test (example #1)
    apoapsis1 = solary.general.astrodyn.kep_apoapsis(sem_maj_axis=5.0, ecc=0.3)
    assert apoapsis1 == 6.5

    # Compute the Apoapsis and perform assertion test (example #2)
    apoapsis2 = solary.general.astrodyn.kep_apoapsis(sem_maj_axis=10.0, ecc=0.0)
    assert apoapsis2 == 10


def test_kep_periapsis():
    """
    Test function for the Periapsis computation.

    Returns
    -------
    None.

    """

    # Compute the Periapsis and perform assertion test (example #1)
    periapsis1 = solary.general.astrodyn.kep_periapsis(sem_maj_axis=5.0, ecc=0.3)
    assert periapsis1 == 3.5

    # Compute the Periapsis and perform assertion test (example #2)
    periapsis2 = solary.general.astrodyn.kep_periapsis(sem_maj_axis=10.0, ecc=0.0)
    assert periapsis2 == 10


def test_mjd2jd():
    """
    Test function to verify Modified Julian Date to Julian Date converter function.

    Returns
    -------
    None.

    """

    # Compute the JD with a given MJD
    jd1 = solary.general.astrodyn.mjd2jd(m_juldate=56123.5)
    assert jd1 == 2456124


def test_jd2mjd():
    """
    Test function to verify Julian Date to Modified Julian Date converter function.

    Returns
    -------
    None.

    """

    # Compute the MJD with a given JD
    mjd1 = solary.general.astrodyn.jd2mjd(juldate=2456000.5)
    assert mjd1 == 56000.0


def test_sphere_of_influence():
    """
    Test function to check the Sphere Of Influence (SOI) computation function.

    Returns
    -------
    None.

    """


    # Read the constants config file and get the value for 1 AU, grav. constant, Earth's and Sun's
    # grav. constant (and convert both to mass)
    config = solary.auxiliary.config.get_constants()
    sem_maj_axis_earth = float(config['constants']['one_au'])
    grav_const = float(config['constants']['grav_const'])
    earth_mass = float(config['constants']['gm_earth']) / grav_const
    sun_mass = float(config['constants']['gm_sun']) / grav_const

    # Compute the SOI of planet Earth
    soi_res_earth = solary.general.astrodyn.sphere_of_influence(sem_maj_axis=sem_maj_axis_earth, \
                                                                minor_mass=earth_mass, \
                                                                major_mass=sun_mass)

    # Assertion test with the SOI's expectation
    assert pytest.approx(soi_res_earth, abs=1e4) == 925000.0


def test_orbit(test_orbit_data):
    """
    Test function to check the orbit base class

    Parameters
    ----------
    test_orbit_data : tuple
        Tuple that contains 2 dictionaries; the orbit values and units.

    Returns
    -------
    None.

    """

    # Check if the fixture is loaded correctly
    assert isinstance(test_orbit_data, tuple)

    # Split the tuple into the 2 dictionaries
    test_orbit_values, test_orbit_units = test_orbit_data

    # Initiate the class
    test_orbit_class = solary.general.astrodyn.Orbit(orbit_values=test_orbit_values,
                                                     orbit_units=test_orbit_units)

    # Check if the instances of the class correspond with the pre-defined settings
    assert test_orbit_class.peri == test_orbit_values['peri']
    assert test_orbit_class.ecc == test_orbit_values['ecc']
    assert test_orbit_class.incl == test_orbit_values['incl']
    assert test_orbit_class.long_asc_node == test_orbit_values['long_asc_node']
    assert test_orbit_class.arg_peri == test_orbit_values['arg_peri']

    # Check the property: semi major axis
    assert test_orbit_class.semi_maj_axis == test_orbit_values['peri'] \
                                             / (1.0 - test_orbit_values['ecc'])

    # Check the property: apoapsis
    assert test_orbit_class.apo == (1.0 + test_orbit_values['ecc']) \
                                   * test_orbit_class.semi_maj_axis



"""
test_camera.py

Testing suite for solary/instruments/camera.py

"""

# Import installed libraries
import pytest

# Import solary
import solary


# Define a test fixture that is being used in all tests. The fixture loads configuration files.
@pytest.fixture(name='ccd_test_config')
def fixture_ccd_test_config():
    """
    Fixture to load the test configuration files.

    Returns
    -------
    test_ccd_dict : dict
        CCD test configuration.

    """

    # Get the test config file paths
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    # Get the path to the CCD config file
    test_ccd_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['instruments_camera_ccd']['properties'])

    # Read and parse the CCD config file and return a dictionary with the properties
    test_ccd_dict = solary.instruments.camera.read_ccd_config(test_ccd_path)

    return test_ccd_dict


def test_read_ccd_config(ccd_test_config):
    """
    Testing if the config file reading was successful.

    Parameters
    ----------
    ccd_test_config : dict
        CCD config dictionary.

    Returns
    -------
    None.

    """

    # Check if the fixture load is a dictionary
    assert isinstance(ccd_test_config, dict)

    # Check the pixels
    assert ccd_test_config['pixels'] == [4096, 4112]

def test_ccd(ccd_test_config):
    """
    Testing the CCD Class.

    Parameters
    ----------
    ccd_test_config : dict
        CCD config dictionary.

    Returns
    -------
    None.

    """

    # Initiate the CCD class
    test_ccd_class = solary.instruments.camera.CCD(ccd_test_config)

    # Check the config depending attributes
    assert test_ccd_class.pixels == ccd_test_config['pixels']
    assert test_ccd_class.pixel_size == ccd_test_config['pixel_size']
    assert test_ccd_class.dark_noise == ccd_test_config['dark_noise']
    assert test_ccd_class.readout_noise == ccd_test_config['readout_noise']
    assert test_ccd_class.full_well == ccd_test_config['full_well']
    assert test_ccd_class.quantum_eff == ccd_test_config['quantum_eff']

    # Check the first entry of the chip size. Multiply the number of pixels (x dimension) times the
    # pixel size in micro meters. Finally divide by 1000 to get a result in mm.
    assert test_ccd_class.chip_size[0] == ccd_test_config['pixels'][0] \
                                          * ccd_test_config['pixel_size'] / 1000.0

    # Check the size of a single pixel in m^2. The pixel size squared leads to micro meter squared.
    # Divide the results by 10^-12.
    assert test_ccd_class.pixel_size_sq_m == (ccd_test_config['pixel_size'] ** 2.0) \
                                             * 10.0 ** (-12.0)



"""
test_geometry.py

Testing suite for solary/general/geometry.py

"""

# Import standard libraries
import math

# Import solary
import solary


def test_circle_area():
    """
    Testing the area computation of a perfect circle.

    Returns
    -------
    None.

    """

    # Compute the area of a unit circle
    circle_area_res1 = solary.general.geometry.circle_area(radius=1.0)
    assert circle_area_res1 == math.pi

    # Second example
    circle_area_res1 = solary.general.geometry.circle_area(radius=2.0)
    assert circle_area_res1 == math.pi * 4.0

def test_fwhm2std():
    """
    Testing the FWHM to standard deviation computation

    Returns
    -------
    None.

    """

    # Compute the standard deviation corresponding FWHM and vice versa
    sigma1_exp = 5.0
    fwhm1 = 2.0 * sigma1_exp * math.sqrt(2.0 * math.log(2))

    sigma1_res = solary.general.geometry.fwhm2std(fwhm1)
    assert sigma1_res == sigma1_exp



"""
physp.py

This script contains miscellaneous functions to describe and derive physical and instrinsic
parameters of asteroids.

"""

# Import standard modules
import math


def ast_size(albedo, abs_mag):
    """
    Compute the radius of an asteroid by using the asteroid's albedo and absolute magnitude.

    Parameters
    ----------
    albedo : float
        Albedo of the object ranging within the intervall (0, 1].
    abs_mag : float
        Absolute magnitude of the object.

    Returns
    -------
    radius : float
        Radius of the object given in kilometer.

    References
    ----------
    [1] Chesley, Steven R.; Chodas, Paul W.; Milani, Andrea; Valsecchi, Giovanni B.; Yeomans,
        Donald K. (October 2002). Quantifying the Risk Posed by Potential Earth Impacts. Icarus.
        159 (2): 425
    [2] https://cneos.jpl.nasa.gov/tools/ast_size_est.html
    [3] http://www.physics.sfasu.edu/astro/asteroids/sizemagnitude.html

    Examples
    --------
    >>> import solary
    >>> ast_radius = solary.asteroid.physp.ast_size(albedo=0.15, abs_mag=10)
    >>> ast_radius
    17.157

    """

    # Compute the diameter in km
    diameter = (1329.0 / math.sqrt(albedo)) * 10.0 ** (-0.2 * abs_mag)

    # Convert the diameter to radius
    radius = diameter / 2.0

    return radius



"""
config.py

Auxiliary functions for all library relevant configuration files

"""

# Import standard libraries
import configparser
import os

# Import the ROOT directory of SolarY
from solary import ROOT_DIR


def get_constants():
    """
    Function to get the constants.ini file from the _config directory

    Returns
    -------
    config : configparser.ConfigParser
        Configuration Parser that contains miscellaneous constants (like astrodynmical, time,
        etc.)

    """

    # Set config parser
    config = configparser.ConfigParser()

    # Get the constants ini file
    constants_ini_path = os.path.join(ROOT_DIR, '_config', 'constants.ini')

    # Read and parse the config file
    config.read(constants_ini_path)

    return config

def get_paths(test=False):
    """
    Function to get the paths.dir file from the _config directory

    Parameters
    ----------
    test : bool
        Boolean value whether to use the default (prod.) or test configs. Default: False

    Returns
    -------
    config : configparser.ConfigParser
        Configuration Parser that contains miscellaneous paths to store / access / etc. downloaded
        files, created database etc.

    """

    # Set the config parser
    config = configparser.ConfigParser()

    # Get the paths ini file, differentiate between prod and test
    if test:
        paths_ini_path = os.path.join(ROOT_DIR, 'tests/_resources/_config', 'test_paths.ini')
    else:
        paths_ini_path = os.path.join(ROOT_DIR, '_config', 'paths.ini')

    # Read and parse the config file
    config.read(paths_ini_path)

    return config



"""
data.py

NEO data download, parsing and database creation functions are part of this sub-module.

"""

# Import standard libraries
import gzip
import urllib.parse
import urllib.request
import os
import re
import shutil
import sqlite3
import time

# Import SolarY
import solary

# Get the file paths
PATH_CONFIG = solary.auxiliary.config.get_paths()

def _get_neodys_neo_nr():
    """
    This function gets the number of currently known NEOs from the NEODyS webpage. The information
    is obtained by a Crawler-like script that may need frequent maintenance.

    Returns
    -------
    neodys_nr_neos : int
        Number of catalogued NEOs in the NEODyS database.

    """

    # Open the NEODyS link, where the current number of NEOs is shown
    http_response = urllib.request.urlopen('https://newton.spacedys.com/neodys/index.php?pc=1.0')

    # Get the HTML response and read its content
    html_content = http_response.read()

    # Extract the number of NEOs from a specific HTML position, using a regular expression. The
    # number is displayed in bold like "[...] <b>1000 objects</b> in the NEODys [...]"
    neodys_nr_neos = int(re.findall(r'<b>(.*?) objects</b> in the NEODyS', str(html_content))[0])

    return neodys_nr_neos


def download(row_exp=None):
    """
    This function downloads a file with the orbital elements of currently all known NEOs from the
    NEODyS database. The file has the ending .cat and is basically a csv / ascii formatted file
    that can be read by any editor. See -1-

    Parameters
    ----------
    row_exp : bool, optional
        Boolean value. If the input is set to True the number of NEOs that are listed in the
        downloaded file are compared with the number of expected NEOs (number of NEOs listed on the
        NEODyS page). The default is None.

    Returns
    -------
    dl_status : str
        Human-readable status report that returns 'OK' or 'ERROR', depending on the download's
        success.
    neodys_neo_nr : int
        Number of NEOs (from the NEODyS page and thus optional). This value can be compared with
        the content of the downloaded file to determine whether entries are missing or not. Per
        default None is returned.

    References
    ----------
    -1- Link to the NEODyS data: https://newton.spacedys.com/neodys/index.php?pc=1.0

    """

    # Set the complete filepath. The file is stored in the user's home directory
    download_filename = \
        solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['neodys_raw_dir'], \
                                                 PATH_CONFIG['neo']['neodys_raw_file'])

    # Download the file
    downl_file_path, _ = \
        urllib.request.urlretrieve(url='https://newton.spacedys.com/~neodys2/neodys.cat', \
                                   filename=download_filename)

    # To check whether the file has been successfully updated (if a file was present before) one
    # needs to compare the "last modification time" of the file with the current system's time. If
    # the deviation is too high, it is very likely that no new file has been downloaded and
    # consequently updated
    system_time = time.time()
    file_mod_time = os.path.getmtime(downl_file_path)
    file_mod_diff = file_mod_time - system_time

    # Set status message, if the file has been updated or not. A time difference of less than 5 s
    # shall indicate whether the file is new or not
    if file_mod_diff < 5:
        dl_status = 'OK'
    else:
        dl_status = 'ERROR'

    # Optional: Get the number of expected NEOs from the NEODyS webpage
    neodys_neo_nr = None
    if row_exp:
        neodys_neo_nr = _get_neodys_neo_nr()

    return dl_status, neodys_neo_nr


def read_neodys():
    """
    Read the content of the downloaded NEODyS file and return a dictionary with its content.

    Returns
    -------
    neo_dict : list
        List of dictionaries that contains the NEO data from the NEODyS download.

    """

    # Set the download file path. The file shall be stored in the home direoctry
    path_filename = solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['neodys_raw_dir'], \
                                                             PATH_CONFIG['neo']['neodys_raw_file'])

    # Set a placeholder dictionary where the data will be stored
    neo_dict = []

    # Open the NEODyS file. Ignore the header (first 6 rows) and iterate through the file row-wise.
    # Read the content adn save it in the dictionary
    with open(path_filename) as f_temp:
        neo_data = f_temp.readlines()[6:]
        for neo_data_line_f in neo_data:
            neo_data_line = neo_data_line_f.split()
            neo_dict.append({'Name': neo_data_line[0].replace('\'', ''), \
                             'Epoch_MJD': float(neo_data_line[1]), \
                             'SemMajAxis_AU': float(neo_data_line[2]), \
                             'Ecc_': float(neo_data_line[3]), \
                             'Incl_deg': float(neo_data_line[4]), \
                             'LongAscNode_deg': float(neo_data_line[5]), \
                             'ArgP_deg': float(neo_data_line[6]), \
                             'MeanAnom_deg': float(neo_data_line[7]), \
                             'AbsMag_': float(neo_data_line[8]), \
                             'SlopeParamG_': float(neo_data_line[9])})

    return neo_dict


class NEOdysDatabase:
    """
    Class to create, update and read an SQLite based database that contains NEO data (raw and
    derived parameters) based on the NEODyS data.

    Attributes
    ----------
    db_filename : str
        Absolute path to the SQLite NEODyS database
    con : sqlite3.Connection
        Connection to the SQLite NEODyS database
    cur: sqlite3.Cursor
        Cursor to the SQLite NEODyS database

    Methods
    -------
    __init__(new=False)
        Init function at the class call. Allows one to re-create a new SQLite database from
        scratch.
    create()
        Create the main table of the SQLite NEODyS database (contains only the raw input data, no
        derived parameters).
    create_deriv_orb()
        Compute derived orbital elements from the raw input data.
    close()
        Close the SQLite database.

    See also
    --------
    solary.neo.data.download(row_exp=None)
    solary.neo.data.read_neodys()

    """


    def __init__(self, new=False):
        """
        Init. function of the NEODySDatabase class. This method creates a new database or opens an
        existing one (if applicable) and sets a cursor.

        Parameters
        ----------
        new : bool, optional
            If True: a new database will be created from scratch. WARNING: this will delete any
            previously built SQLite database with the name "neo_neodys.db" in the home directory.
            The default is False.

        """

        # Set / Get an SQLite database path + filename
        self.db_filename = \
            solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['neodys_db_dir'], \
                                                     PATH_CONFIG['neo']['neodys_db_file'])

        # Delete any existing database, if requested
        if new and os.path.exists(self.db_filename):
            os.remove(self.db_filename)

        # Connect / Build database and set a cursor
        self.con = sqlite3.connect(self.db_filename)
        self.cur = self.con.cursor()


    def _create_col(self, table, col_name, col_type):
        """
        Private method to create new columns in tables

        Parameters
        ----------
        table : str
            Table name, where a new column shall be added.
        col_name : str
            Column name.
        col_type : str
            SQLite column type (FLOAT, INT, TEXT, etc.).

        """

        # Generic f-string that represents an SQLite command to alter a table (adding a new column
        # with its dtype).
        sql_col_create = f'ALTER TABLE {table} ADD COLUMN {col_name} {col_type}'

        # Try to create a new column. If is exists an sqlite3.OperationalError weill raise. Pass
        # this error.
        #TODO: change the error passing
        try:
            self.cur.execute(sql_col_create)
            self.con.commit()
        except sqlite3.OperationalError:
            pass


    def create(self):
        """
        Method to create the NEODyS main table, read the downloaded content and fill the database
        with the raw data.

        """

        # Create the main table
        self.cur.execute('CREATE TABLE IF NOT EXISTS main(Name TEXT PRIMARY KEY, ' \
                                                         'Epoch_MJD FLOAT, ' \
                                                         'SemMajAxis_AU FLOAT, ' \
                                                         'Ecc_ FLOAT, ' \
                                                         'Incl_deg FLOAT, ' \
                                                         'LongAscNode_deg FLOAT, ' \
                                                         'ArgP_deg FLOAT, ' \
                                                         'MeanAnom_deg FLOAT, ' \
                                                         'AbsMag_ FLOAT, ' \
                                                         'SlopeParamG_ FLOAT)')
        self.con.commit()

        # Read the NEODyS raw data
        _neo_data = read_neodys()

        # Insert the raw data into the database
        self.cur.executemany('INSERT OR IGNORE INTO main(Name, ' \
                                                        'Epoch_MJD, ' \
                                                        'SemMajAxis_AU, ' \
                                                        'Ecc_, ' \
                                                        'Incl_deg, ' \
                                                        'LongAscNode_deg, ' \
                                                        'ArgP_deg, ' \
                                                        'MeanAnom_deg, ' \
                                                        'AbsMag_, ' \
                                                        'SlopeParamG_) ' \
                                                    'VALUES (:Name, ' \
                                                            ':Epoch_MJD, ' \
                                                            ':SemMajAxis_AU, ' \
                                                            ':Ecc_, ' \
                                                            ':Incl_deg, ' \
                                                            ':LongAscNode_deg, ' \
                                                            ':ArgP_deg, ' \
                                                            ':MeanAnom_deg, ' \
                                                            ':AbsMag_, ' \
                                                            ':SlopeParamG_)', \
                             _neo_data)
        self.con.commit()


    def create_deriv_orb(self):
        """
        Method to compute and insert derived orbital elements into the SQLite database.

        """

        # Add new columns in the main table
        self._create_col('main', 'Aphel_AU', 'FLOAT')
        self._create_col('main', 'Perihel_AU', 'FLOAT')

        # Get orbital elements to compute the derived parameters
        self.cur.execute('SELECT Name, SemMajAxis_AU, Ecc_ FROM main')

        # Fetch the data
        _neo_data = self.cur.fetchall()

        # Iterate throuh the results, compute the derived elements and put them in a list of
        # dicitionaries
        _neo_deriv_param_dict = []
        for _neo_data_line_f in _neo_data:
            _neo_deriv_param_dict.append({'Name': _neo_data_line_f[0], \
                                          'Aphel_AU': \
                                              solary.general.astrodyn.kep_apoapsis( \
                                                                sem_maj_axis=_neo_data_line_f[1], \
                                                                ecc=_neo_data_line_f[2] \
                                                                                  ), \
                                          'Perihel_AU': \
                                              solary.general.astrodyn.kep_periapsis( \
                                                                sem_maj_axis=_neo_data_line_f[1], \
                                                                ecc=_neo_data_line_f[2] \
                                                                                  )})

        # Insert the data into the main table
        self.cur.executemany('UPDATE main SET Aphel_AU = :Aphel_AU, Perihel_AU = :Perihel_AU ' \
                             'WHERE Name = :Name', _neo_deriv_param_dict)
        self.con.commit()


    def update(self):
        """
        Update the NEODyS Database with all content.

        """

        # Call the create functions that insert new data
        self.create()
        self.create_deriv_orb()


    def close(self):
        """
        Close the SQLite NEODyS database.

        """

        self.con.close()


def download_granvik2018():
    """
    Function to download the model data from Granvik et al. (2018) -1-. The data can be found in
    -2-.

    Returns
    -------
    md5_hash : str
        MD5 hash of the downloaded file.

    References
    ----------
    -1- Granvik, Morbidelli, Jedicke, Bolin, Bottke, Beshore, Vokrouhlicky, Nesvorny, and Michel
        (2018). Debiased orbit and absolute-magnitude distributions for near-Earth objects.
        Accepted for publication in Icarus.
    -2- https://www.mv.helsinki.fi/home/mgranvik/data/Granvik+_2018_Icarus/
    """

    # Set the download path to the home directory
    download_filename = \
        solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['granvik2018_raw_dir'], \
                                                 PATH_CONFIG['neo']['granvik2018_raw_file'])

    # Set the downlaod URL
    url_location = 'https://www.mv.helsinki.fi/home/mgranvik/data/' \
                   'Granvik+_2018_Icarus/Granvik+_2018_Icarus.dat.gz'

    # Retrieve the data (download)
    downl_file_path, _ = urllib.request.urlretrieve(url=url_location, \
                                                    filename=download_filename)

    # Get the file name (without the gzip ending). Open the gzip file and move the .dat file out.
    unzip_file_path = downl_file_path[:-3]
    with gzip.open(downl_file_path, 'r') as f_in, open(unzip_file_path, 'wb') as f_out:
        shutil.copyfileobj(f_in, f_out)

    # Delete the gzip file
    os.remove(downl_file_path)

    # Compute the MD5 hash
    md5_hash = solary.auxiliary.parse.comp_md5(unzip_file_path)

    return md5_hash


def read_granvik2018():
    """
    Read the content of the downloaded Granvik et al. (2018) NEO model data file and return a
    dictionary with its content.

    Returns
    -------
    neo_dict : list
        List of dictionaries that contains the NEO data from the downloaded model data.

    """

    # Set the download path of the model file
    path_filename = \
        solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['granvik2018_raw_dir'], \
                                                 PATH_CONFIG['neo']['granvik2018_unzip_file'])

    # Iterate through the downloaded file and write the content in a list of dictionaries. Each
    # dictionary contains an individual simulated NEO
    neo_dict = []
    with open(path_filename) as f_temp:
        neo_data = f_temp.readlines()

        for neo_data_line_f in neo_data:
            neo_data_line = neo_data_line_f.split()
            neo_dict.append({'SemMajAxis_AU': float(neo_data_line[0]), \
                             'Ecc_': float(neo_data_line[1]), \
                             'Incl_deg': float(neo_data_line[2]), \
                             'LongAscNode_deg': float(neo_data_line[3]), \
                             'ArgP_deg': float(neo_data_line[4]), \
                             'MeanAnom_deg': float(neo_data_line[5]), \
                             'AbsMag_': float(neo_data_line[6])})

    return neo_dict


class Granvik2018Database:
    """
    Class to create, update and read an SQLite based database that contains the Granvik et al.
    (2018) NEO model data (raw and derived parameters)

    Attributes
    ----------
    db_filename : str
        Absolute path to the SQLite Granvik et al. (2018) database
    con : sqlite3.Connection
        Connection to the SQLite Granvik et al. (2018) database
    cur: sqlite3.Cursor
        Cursor to the SQLite Granvik et al. (2018) database

    Methods
    -------
    __init__(new=False)
        Init function at the class call. Allows one to re-create a new SQLite database from
        scratch.
    create()
        Create the main table of the SQLite Granvik et al. (2018) database (contains only the raw
        input data, no derived parameters).
    create_deriv_orb()
        Compute derived orbital elements from the raw input data.
    close()
        Close the SQLite database.

    See also
    --------
    solary.neo.data.download_granvik2018()
    solary.neo.data.read_granvik2018()

    """


    def __init__(self, new=False):
        """
        Init. function of the Granvik2018Database class. This method creates a new database or
        opens an existing one (if applicable) and sets a cursor.

        Parameters
        ----------
        new : bool, optional
            If True: a new database will be created from scratch. WARNING: this will delete any
            previously built SQLite database with the name "neo_granvik2018.db" in the home
            directory. The default is False.

        """

        # Set the database path to the home directory
        self.db_filename = \
            solary.auxiliary.parse.setnget_file_path(PATH_CONFIG['neo']['granvik2018_db_dir'], \
                                                     PATH_CONFIG['neo']['granvik2018_db_file'])

        # Delete any existing database, if requested
        if new and os.path.exists(self.db_filename):
            os.remove(self.db_filename)

        # Establish a connection and set a cursor to the database
        self.con = sqlite3.connect(self.db_filename)
        self.cur = self.con.cursor()


    def _create_col(self, table, col_name, col_type):
        """
        Private method to create new columns in tables

        Parameters
        ----------
        table : str
            Table name, where a new column shall be added.
        col_name : str
            Column name.
        col_type : str
            SQLite column type (FLOAT, INT, TEXT, etc.).

        """

        # Generic f-string that represents an SQLite command to alter a table (adding a new column
        # with its dtype).
        sql_col_create = f'ALTER TABLE {table} ADD COLUMN {col_name} {col_type}'

        # Try to create a new column. If is exists an sqlite3.OperationalError weill raise. Pass
        # this error.
        #TODO: change the error passing and merge with the same method in NEODySDatabase
        try:
            self.cur.execute(sql_col_create)
            self.con.commit()
        except sqlite3.OperationalError:
            pass


    def create(self):
        """
        Method to create the Granvik et al. (2018) main table, read the downloaded content and fill
        the database with the raw data.

        """

        # Create main table for the raw data
        self.cur.execute('CREATE TABLE IF NOT EXISTS main(ID INTEGER PRIMARY KEY, ' \
                                                         'SemMajAxis_AU FLOAT, ' \
                                                         'Ecc_ FLOAT, ' \
                                                         'Incl_deg FLOAT, ' \
                                                         'LongAscNode_deg FLOAT, ' \
                                                         'ArgP_deg FLOAT, ' \
                                                         'MeanAnom_deg FLOAT, ' \
                                                         'AbsMag_ FLOAT)')
        self.con.commit()

        # Read the Granvik et al. (2018) data
        _neo_data = read_granvik2018()

        # Insert the raw Granvik et al. (2018) data into the SQLite database
        self.cur.executemany('INSERT OR IGNORE INTO main(SemMajAxis_AU, ' \
                                                        'Ecc_, ' \
                                                        'Incl_deg, ' \
                                                        'LongAscNode_deg, ' \
                                                        'ArgP_deg, ' \
                                                        'MeanAnom_deg, ' \
                                                        'AbsMag_) ' \
                                                    'VALUES (:SemMajAxis_AU, ' \
                                                            ':Ecc_, ' \
                                                            ':Incl_deg, ' \
                                                            ':LongAscNode_deg, ' \
                                                            ':ArgP_deg, ' \
                                                            ':MeanAnom_deg, ' \
                                                            ':AbsMag_)', \
                             _neo_data)
        self.con.commit()


    def create_deriv_orb(self):
        """
        Method to compute and insert derived orbital elements into the SQLite database.

        """

        # Create new columns
        self._create_col('main', 'Aphel_AU', 'FLOAT')
        self._create_col('main', 'Perihel_AU', 'FLOAT')

        # Get all relevant information from the database
        self.cur.execute('SELECT ID, SemMajAxis_AU, Ecc_ FROM main')
        _neo_data = self.cur.fetchall()


        # Iterate through the results and compute the derived orbital parameters
        _neo_deriv_param_dict = []
        for _neo_data_line_f in _neo_data:
            _neo_deriv_param_dict.append({'ID': _neo_data_line_f[0], \
                                          'Aphel_AU': \
                                              solary.general.astrodyn.kep_apoapsis( \
                                                                sem_maj_axis=_neo_data_line_f[1], \
                                                                ecc=_neo_data_line_f[2] \
                                                                                  ), \
                                          'Perihel_AU': \
                                              solary.general.astrodyn.kep_periapsis( \
                                                                sem_maj_axis=_neo_data_line_f[1], \
                                                                ecc=_neo_data_line_f[2] \
                                                                                  )})

        # Insert the derived paramters into the database
        self.cur.executemany('UPDATE main SET Aphel_AU = :Aphel_AU, Perihel_AU = :Perihel_AU ' \
                             'WHERE ID = :ID', _neo_deriv_param_dict)
        self.con.commit()


    def close(self):
        """
        Close the Granvik et al. (2018) database.

        """

        self.con.close()



"""
astrodyn.py

Miscellaneous functions regarding astro-dynamical topics can be found here.

"""

# Import standard modules
import math

# Import solary
import solary


def tisserand(sem_maj_axis_obj, inc, ecc, sem_maj_axis_planet=None):
    """
    Compute the Tisserand parameter of an object w.r.t. a larger object. If no semi-major axis of
    a larger object is given, the values for Jupiter are assumed.

    Parameters
    ----------
    sem_maj_axis_obj : float
        Semi-major axis of the minor object (whose Tisserand parameter shall be computed) given in
        AU.
    inc : float
        Inclination of the minor object given in radians.
    ecc : float
        Eccentricity of the minor object.
    sem_maj_axis_planet : float, optional
        Semi-major axis of the major object. If no value is given, the semi-major axis of Jupiter
        is taken. The default is None.

    Returns
    -------
    tisserand_parameter : float
        Tisserand parameter of the minor object w.r.t. the major object.

    Notes
    -----
    The Tisserand parameter provides a dimensionless value for the astro-dyamical relation between
    e.g., comets and Jupiter. Let's assume one wants to compute the Tisserand parameter of a comet
    with the largest gas giant. A Tisserand parameter between 2 and 3 (2 < Tisserand < 3) indicates
    a so called Jupiter-Family Comet (JFC).

    Examples
    --------
    An example to compute the Tisserand parameter of the comet 67P/Churyumov–Gerasimenko. The data
    have been obtained from https://ssd.jpl.nasa.gov/sbdb.cgi?sstr=67P

    >>> import math
    >>> import solary
    >>> tisserand_tsch_geras_67p = solary.general.astrodyn.tisserand(sem_maj_axis_obj=3.46, \
                                                                     inc=math.radians(7.03), \
                                                                     ecc=0.64)
    >>> tisserand_tsch_geras_67p
    2.747580043374075

    """

    # If no semi-major axis of a larger object is given: Assume the planet Jupiter. Jupiter's
    # semi-major axis can be found in the config file
    if not sem_maj_axis_planet:

        # Get the constants config file
        config = solary.auxiliary.config.get_constants()
        sem_maj_axis_planet = float(config['planets']['sem_maj_axis_jup'])

    # Compute the tisserand parameter
    tisserand_parameter = (sem_maj_axis_planet / sem_maj_axis_obj) + 2.0 * math.cos(inc) \
                          * math.sqrt((sem_maj_axis_obj / sem_maj_axis_planet) * (1.0 - ecc**2.0))

    return tisserand_parameter


def kep_apoapsis(sem_maj_axis, ecc):
    """
    Compute the apoapsis, depending on the semi-major axis and eccentricity.

    Parameters
    ----------
    sem_maj_axis : float
        Semi-major axis of the object in any unit.
    ecc : float
        Eccentricity of the object.

    Returns
    -------
    apoapsis : float
        Apoapsis of the object. Unit is identical to input unit of sem_maj_axis.

    """

    apoapsis = (1.0 + ecc) * sem_maj_axis

    return apoapsis


def kep_periapsis(sem_maj_axis, ecc):
    """
    Compute the periapsis, depending on the semi-major axis and eccentricity.

    Parameters
    ----------
    sem_maj_axis : float
                Semi-major axis of the object in any unit.
    ecc : float
        Eccentricity of the object.

    Returns
    -------
    periapsis : float
        Periapsis of the object. Unit is identical to input unit of sem_maj_axis.

    """

    periapsis = (1.0 - ecc) * sem_maj_axis

    return periapsis


def mjd2jd(m_juldate):
    """
    Convert the given Julian Date to the Modified Julian Date.

    Parameters
    ----------
    m_juldate : float
        Modified Julian Date.

    Returns
    -------
    juldate : float
        Julian Date.

    """

    juldate = m_juldate + 2400000.5

    return juldate


def jd2mjd(juldate):
    """
    Convert the Modified Julian Date to the Julian Date.

    Parameters
    ----------
    juldate : float
        Julian Date.

    Returns
    -------
    m_juldate : float
        Modified Julian Date.

    """

    m_juldate = juldate - 2400000.5

    return m_juldate


def sphere_of_influence(sem_maj_axis, minor_mass, major_mass):
    """
    Compute the Sphere of Influence (SOI) of a minor object w.r.t. a major object, assuming a
    spherical SOI

    Parameters
    ----------
    sem_maj_axis : float
        Semi-Major Axis given in any physical dimension.
    minor_mass : float
        Mass of the minor object given in any physical dimension.
    major_mass : float
        Mass of the major object given in the same physical dimension as minor_mass.

    Returns
    -------
    soi_radius : float
        SOI radius given in the same physical dimension as sem_maj_axis.

    """

    # Compute the Sphere of Influence (SOI)
    soi_radius = sem_maj_axis * ((minor_mass / major_mass) ** (2.0 / 5.0))

    return soi_radius


class Orbit:
    """
    The Orbit class is a base class for further classes. Basic orbital computations are performed
    that are object type agnostic (like computating the apoapsis of an object). The base class
    requires the values of an orbit as well as the corresponding units for the spatial and angle
    information.

    Attributes
    ----------
    peri : float
        Periapsis. Given in any spatial dimension.
    ecc : float
        Eccentricity.
    incl : float
        Inclination. Given in degrees or radians.
    long_asc_node : float
        Longitude of ascending node. Given in the same units as incl.
    arg_peri : float
        Argument of periapsis. Given in the same uniuts as incl.
    units_dict : dict
        Dictionary that contains the keys "spatial" and "angle" that provide the units "km", "AU"
        and "rad" and "deg" respectively.

    Static Properties
    -----------------
    sem_maj_axis : float
        Semi-major axis. Given in the same units as peri.
    apo : float
        Apoapsis. Given in the same units as peri.

    """


    def __init__(self, orbit_values, orbit_units):
        """
        Init function.

        Parameters
        ----------
        orbit_values : dict
            Dictionary that contains the orbit's values. The spatial and angle unit are given by
            the orbit_units dictionary
        orbit_units : dict
            Dictionary that contains the orbit's values corresponding units (spatial and angle).

        Returns
        -------
        None.

        """

        # Setting attribute placeholders
        self.peri = None
        self.ecc = None
        self.incl = None
        self.long_asc_node = None
        self.arg_peri = None

        # Set the units dictionary
        self.units_dict = orbit_units

        # Set the attributes dynamically in a for loop and set the values accordingly.
        valid_keys = ['peri', 'ecc', 'incl', 'long_asc_node', 'arg_peri']
        for key in valid_keys:
            setattr(self, key, orbit_values.get(key))


    @property
    def semi_maj_axis(self):
        """
        Get the semi-major axis

        Returns
        -------
        _semi_maj_axis : float
            Semi-major axis. Given in the same spatial dimension as the input parameters.

        """

        # Compute the semi-major axis
        _semi_maj_axis = self.peri / (1.0-self.ecc)

        return _semi_maj_axis


    @property
    def apo(self):
        """
        Get the apoapsis

        Returns
        -------
        _apo : float
            Apoapsis. Given in the the same spatial dimension as the input parameters.

        """

        # Comput the apoapsis
        _apo = kep_apoapsis(sem_maj_axis=self.semi_maj_axis, ecc=self.ecc)

        return _apo



"""
reader.py

Auxiliary reader for miscellaneous files.

"""

# Import standard libraries
import json

def read_orbit(orbit_path):
    """
    Function to read an orbit properties file.

    Parameters
    ----------
    orbit_path : str
        File path of the JSON file that contains the properties information.

    Returns
    -------
    orbit_values : dict
        Orbit values.
    orbit_units : dict
        Orbit units.

    """

    # Open the file path and load / read it as a JSON
    with open(orbit_path) as temp_obj:
        orbit_compl = json.load(temp_obj)

    #TODO: check if values and units are in json and then check if the keys are correct

    # Get the values and units dictionaries within the JSON file
    orbit_values = orbit_compl['values']
    orbit_units = orbit_compl['units']

    return orbit_values, orbit_units



# pylint: disable=no-member
"""
optics.py

This script contains classes and functions that are needed for optical systems.

"""

# Import standard libraries
import json

# Import solary
import solary


def read_reflector_config(config_filepath):
    """
    Function to read the configuration file for a reflector system.

    Parameters
    ----------
    config_filepath : str
        File path of the configuration file.

    Returns
    -------
    optics_config : dict
        Reflector configuration.

    """

    # Open the file path and load / read it as a JSON
    with open(config_filepath) as temp_obj:
        optics_config = json.load(temp_obj)

    return optics_config


class Reflector:
    """
    Class that defines the optical system of a reflector telescope. The class is a base class for
    a high level telescope class and dictionary configurations to set attributes. Properties are
    derived from the user's input like e.g. the collector area of a telescope.

    Attributes
    ----------
    main_mirror_dia : float
        Diameter of the main mirror. Given in m.
    sec_mirr_dia : float
        Diameter of the secondary mirror. Given in m.
    optical_throughput: float
        Throughput of the telescope. The Throughput is a combination of the mirror reflectivity,
        filter transmissivity etc. It describes only the optical system and does no include e.g.,
        the quantum efficiency of a camera system. Dimensionless and defined between 0 and 1.
    focal_length : float
        Focal length of the system. Given in m.

    Static Properties
    -----------------
    main_mirror_area : float
        Main mirror area, assuming a circular mirror. Given in m^2.
    sec_mirror_area : float
        Secondary mirror area, assuming a cirular mirror. Given in m^2.
    collect_area : float
        Effective photon collection area (main mirror area substracted by the secondary mirror area)

    """


    def __init__(self, optics_config):
        """
        Init function.

        Parameters
        ----------
        optics_config : dict
            Reflector configuration dictionary.

        Returns
        -------
        None.

        """

        # Setting attribute placeholders
        self.main_mirror_dia = None
        self.sec_mirror_dia = None
        self.optical_throughput = None
        self.focal_length = None

        # Set the attributes dynamically in a for loop and set the values accordingly.
        valid_keys = ['main_mirror_dia', 'sec_mirror_dia', 'optical_throughput', 'focal_length']
        for key in valid_keys:
            setattr(self, key, optics_config.get(key))


    @property
    def main_mirror_area(self):
        """
        Get the main mirror area, assuming a circular shaped mirror.

        Returns
        -------
        float
            Main mirror area. Given in m^2.

        """

        # Call a sub-module that requires the radius as an input
        return solary.general.geometry.circle_area(self.main_mirror_dia / 2.0)


    @property
    def sec_mirror_area(self):
        """
        Get the secondary mirror area, assuming a circular shaped mirror.

        Returns
        -------
        float
            Secondary mirror area. Given in m^2.

        """

        # Call a sub-module that requires the radius as an input
        return solary.general.geometry.circle_area(self.sec_mirror_dia / 2.0)


    @property
    def collect_area(self):
        """
        Get the photon collection area. This propery simply substracts the secondary mirror area
        from the main one.

        Returns
        -------
        float
            Photon collection area. Given in m^2.

        """

        return self.main_mirror_area - self.sec_mirror_area



# pylint: disable=W0212
"""
test_telescope.py

Testing suite for solary/instruments/telescope.py

"""

# Import standard libraries
import math

# Import installed libraries
import pytest

# Import solary
import solary


@pytest.fixture(name='telescope_test_properties')
def fixture_telescope_test_properties():
    """
    Fixture to load the test configuration files.


    Returns
    -------
    test_reflector : dict
        Reflector test configuration.
    test_ccd : dict
        CCD test configuration.

    """

    # Get the test config paths
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    # Load and parse the reflector config
    test_reflector_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['instruments_optics_reflector']['properties'])

    test_reflector = solary.instruments.optics.read_reflector_config(test_reflector_path)

    # Load and parse the CCD properties
    test_ccd_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['instruments_camera_ccd']['properties'])

    test_ccd = solary.instruments.camera.read_ccd_config(test_ccd_path)

    return test_reflector, test_ccd


def test_comp_fov():
    """
    Test the Field-Of-View computation function

    Returns
    -------
    None.

    """

    # Test sample with a sensor dimension / size of 25.4 mm and a telescope focial length of 1000
    # mm
    fov1 = solary.instruments.telescope.comp_fov(sensor_dim=25.4,
                                                 focal_length=1000.0)
    assert pytest.approx(fov1 / 60.0, abs=0.1) == 87.3


def test_reflectorccd(telescope_test_properties):
    """
    Testing the telescope class. To compare the rather complex and error-prone computation, a PERL
    based script from [1] has been used for rough computation comparisons.

    Parameters
    ----------
    telescope_test_properties : tuple
        Tuple with 2 dictionaries: reflector config dict and CCD config dict.

    Returns
    -------
    None.

    References
    ----------
    [1] http://spiff.rit.edu/richmond/signal.shtml 04.Jan.2021

    """

    # Set some settings for later use
    photometric_aperture = 10.0
    half_flux_diameter = 10.0
    expos_time = 60.0

    # Set some observational parameters
    object_brightness = 19.0
    sky_brightness = 19.0

    # Check if the property tuple is correctly formatted
    assert isinstance(telescope_test_properties, tuple)
    assert len(telescope_test_properties) == 2

    # Get the config dictionaries
    test_reflector_config, test_ccd_config = telescope_test_properties

    # Initiate the telescope class
    test_telescope = solary.instruments.telescope.ReflectorCCD(test_reflector_config,
                                                               test_ccd_config)

    # Check attributes
    assert test_telescope.pixels == test_ccd_config['pixels']
    assert test_telescope.main_mirror_dia == test_reflector_config['main_mirror_dia']

    # Test if constants config has been loaded
    config = solary.auxiliary.config.get_constants()
    assert test_telescope._photon_flux_v == float(config['photometry']['photon_flux_V'])

    # Test now the telescope specific properties, FOV
    assert pytest.approx(test_telescope.fov[0], abs=0.1) == 1266.8
    assert pytest.approx(test_telescope.fov[1], abs=0.1) == 1271.8

    # Check iFOV
    assert test_telescope.ifov[0] == test_telescope.fov[0] / test_telescope.pixels[0]

    assert test_telescope.ifov[1] == test_telescope.fov[1] / test_telescope.pixels[1]

    # Set aperture in arcsec
    test_telescope.aperture = photometric_aperture
    assert test_telescope.aperture == photometric_aperture

    # Set the half flux diameter in arcsec
    test_telescope.hfdia = half_flux_diameter
    assert test_telescope.hfdia == half_flux_diameter

    # Check how many pixels are aperture
    assert test_telescope.pixels_in_aperture == \
        int(round(solary.general.geometry.circle_area(0.5 * test_telescope.aperture) \
                  / math.prod(test_telescope.ifov), 0))

    # Set exposure time in seconds
    test_telescope.exposure_time = expos_time
    assert test_telescope.exposure_time == expos_time

    # Compute the light fraction in aperture
    assert test_telescope._ratio_light_aperture == \
        math.erf((test_telescope.aperture) \
                 / (solary.general.geometry.fwhm2std(test_telescope.hfdia)*math.sqrt(2)))**2.0

    # Compute raw object electrons (expectation)
    exp_e_signal = round(10.0 ** (-0.4 * object_brightness) \
                         * float(config['photometry']['photon_flux_V']) \
                         * expos_time \
                         * test_telescope.collect_area \
                         * test_telescope.quantum_eff \
                         * test_telescope.optical_throughput \
                         * test_telescope._ratio_light_aperture, 0)

    # Compute the electron signal and compare the results
    object_electrons_apert = test_telescope.object_esignal(mag=object_brightness)
    assert object_electrons_apert == exp_e_signal

    # Compute sky background signal (expectation)
    # Convert surface brightness to integrated brightness over entire FOV
    total_sky_mag = solary.general.photometry.surmag2intmag(surmag=sky_brightness, \
                                                            area=math.prod(test_telescope.fov))
    exp_sky_esignal = round(10.0 ** (-0.4 * total_sky_mag) \
                            * float(config['photometry']['photon_flux_V']) \
                            * expos_time \
                            * test_telescope.collect_area \
                            * test_telescope.quantum_eff \
                            * test_telescope.optical_throughput \
                            * (test_telescope.pixels_in_aperture
                               / math.prod(test_telescope.pixels)), 0)

    # Compare computation with expectation
    sky_electrons_apert = test_telescope.sky_esignal(mag_arcsec_sq=sky_brightness)
    assert sky_electrons_apert == exp_sky_esignal

    # Test the dark current
    assert test_telescope.dark_esignal_aperture == \
        round(float(test_ccd_config['dark_noise']) \
              * expos_time \
              * test_telescope.pixels_in_aperture, 0)

    # Now test the SNR
    assert pytest.approx(test_telescope.object_snr(obj_mag=object_brightness,
                                                   sky_mag_arcsec_sq=sky_brightness),
                                                   abs=0.1) == 5.0



"""
geometry.py

Auxiliary functions for geometric purposes.

"""

# Import standard libraries
import math


def circle_area(radius):
    """
    Compute the area of a perfect with a given radius.

    Parameters
    ----------
    radius : float
        Radius of the circle given in any dimension.

    Returns
    -------
    area : float
        Area of the circle, given in the input dimension^2.

    """

    # Compute the area of a circle
    area = math.pi * (radius ** 2.0)

    return area


def fwhm2std(fwhm):
    """
    Convert the Full Width at Half Maximum to the corresponding Gaussian standard deviation.

    Parameters
    ----------
    fwhm : float
        Full Width at Half Maximum.

    Returns
    -------
    gauss_sigma : float
        Standard deviation assuming a Gaussian distribution.

    """

    # Compute the standard deviation
    gauss_sigma = fwhm / (2.0 * math.sqrt(2.0 * math.log(2)))

    return gauss_sigma



"""
test_physp.py

Testing suite for solary/asteroid/physp

"""

# Import installed modules
import pytest

# Import solary
import solary


def test_ast_size():
    """
    Test function for the asteroid size computation function

    Returns
    -------
    None.

    """

    # Compute the radius of an asteroid (test 1) and compare it with the (approximated expectation)
    ast_radius1 = solary.asteroid.physp.ast_size(albedo=0.05, abs_mag=10.0)
    assert pytest.approx(ast_radius1) == 29.71734

    # Compute the radius of an asteroid (test 2) and compare it with the (approximated expectation)
    ast_radius2 = solary.asteroid.physp.ast_size(albedo=0.3, abs_mag=20.0)
    assert pytest.approx(ast_radius2) == 0.12132055

    # Compute the radius of an asteroid (test 3) and compare it with the (approximated expectation)
    ast_radius3 = solary.asteroid.physp.ast_size(albedo=0.15, abs_mag=26.0)
    assert pytest.approx(ast_radius3) == 0.010825535



"""
camera.py

Script that contains miscellaneous functions and classes for / of camera system (e.g., CCDs).

"""

# Import standard libraries
import json


def read_ccd_config(config_filepath):
    """
    Function to read the configuration file for a CCD system.

    Parameters
    ----------
    config_filepath : str
        File path of the configuration file.

    Returns
    -------
    ccd_config : dict
        CCD configuration.

    """

    # Open the file path and load / read it as a JSON
    with open(config_filepath) as temp_obj:
        ccd_config = json.load(temp_obj)

    return ccd_config


class CCD:
    """
    Class that defines ande describes a CCD camera. Properties are derived from the user's input
    like e.g., the chip size.

    Attributes
    ----------
    pixels : list
        List with the number of pixels per dimension.
    pixel_size : float
        Size of a single pixel, assuming a square shaped pixel. Given in micro meter.
    dark_noise : float
        Dark noise / current of a single pixel. Given in electrons^-1 * s^-1 * pixel^-1.
    readout_noise : float
        Readout noise of a single pixel. Given in electrons^-1 * pixel^-1.
    full_well : int or float
        Number of max. electrons per pixel.

    Static Properties
    -----------------
    chip_size : list
        List with the size of the CCD chip in each dimension (x, y). Given in mm.
    pixel_size_sq_m : float
        Size of a single pixel, assuming a square shaped pixel. Given in m^2.

    """


    def __init__(self, ccd_config):
        """
        Init function.

        Parameters
        ----------
        ccd_config : dict
            CCD configuration dictionary.

        Returns
        -------
        None.

        """

        # Placeholders for the static attributes
        self.pixels = [None, None]
        self.pixel_size = None
        self.dark_noise = None
        self.readout_noise = None
        self.full_well = None
        self.quantum_eff = None

        # Set the attributes dynamically in a for loop and set the values accordingly.
        valid_keys = ['pixels', 'pixel_size', 'dark_noise', 'readout_noise', 'full_well',
                      'quantum_eff']
        for key in valid_keys:
            setattr(self, key, ccd_config.get(key))


    @property
    def chip_size(self):
        """
        Get the chip size (x and y dimension). Given in mm.

        Returns
        -------
        chip_size : list
                    List with the size of the CCD chip in each dimension (x, y). Given in mm.

        """

        # Placeholder list for the results
        chip_size = []

        # Convert the pixel size, given in micro meter, to mm.
        pixel_size_mm = self.pixel_size / 1000.0

        # Compute the chip size in each dimension by multiplying the number of pixels with the
        # pixel size, given in mm.
        for pixel_dim in self.pixels:
            chip_size.append(pixel_dim*pixel_size_mm)

        return chip_size


    @property
    def pixel_size_sq_m(self):
        """
        Get the size of a single pixel in m^2.

        Returns
        -------
        float
            Size of a single pixel. Given in m^2.

        """

        # Conversion between micro meter^2 to meter^2 -> 10^-12
        return (self.pixel_size ** 2.0) * (10.0 ** (-12))



import setuptools

import solary

setuptools.setup(
    name='solary',
    version=solary.__version__,
    author=solary.__author__,
    packages=setuptools.find_packages(),
    package_data={'solary': ['_config/constants.ini', '_config/paths.ini']},
    include_package_data=True,
    description="A Space Science library for asteroid, comets and meteors",
    long_description="A Space Science library for asteroid, comets and meteors",
    url="https://github.com/ThomasAlbin/SolarY",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research"
    ],
    python_requires='>=3.8',
    install_requires=[
        "pytest>=6.1.0",
        "certifi"
    ],
)



"""
test_reader.py

Testing suite for the reader functionality

"""

# Import SolarY
import solary


def test_read_orbit():
    """
    Test the reader function read_orbit.

    Returns
    -------
    None.

    """

    # Get the test config file paths
    test_paths_config = solary.auxiliary.config.get_paths(test=True)

    # Parse the orbit path
    test_orbit_path = \
        solary.auxiliary.parse.get_test_file_path(
            test_paths_config['general_astrodyn']['base_class_orbit'])

    # Read and parse the orbit file and return a values and units dictionary
    test_orbit_values, test_orbit_units = \
        solary.auxiliary.reader.read_orbit(test_orbit_path)

    # Check whether the instances are correct and the expectations
    assert isinstance(test_orbit_values, dict)
    assert isinstance(test_orbit_units, dict)

    assert test_orbit_values['peri'] == 1.133
    assert test_orbit_units['spatial'] == 'AU'



"""
parse.py

Auxiliary functions for file and path parsing

"""

# Import standard libraries
import hashlib
import pathlib
import os

# Get the ROOT DIR
from solary import ROOT_DIR


def comp_md5(file_name):
    """
    Compute the MD5 hash of a file.

    Parameters
    ----------
    file_name : str
        Absolute or relative pathname of the file that shall be parsed.

    Returns
    -------
    md5_res : str
        Resulting MD5 hash.

    """

    # Set the MD5 hashing
    hash_md5 = hashlib.md5()

    # Open the file in binary mode (read-only) and parse it in 65,536 byte chunks (in case of
    # large files, the loading will not exceed the usable RAM)
    with open(file_name, 'rb') as f_temp:
        for _seq in iter(lambda: f_temp.read(65536), b''):
            hash_md5.update(_seq)

    # Digest the MD5 result
    md5_res = hash_md5.hexdigest()

    return md5_res


def setnget_file_path(dl_path, filename):
    """
    Compute the path of a file, depending on its download path. The standard download path is:
        ~$HOME/

    Parameters
    ----------
    dl_path : str
        Relative path of the directory where the file is stored (relative to ~$HOME/solary_data/).
    filename : str
        File name.

    Returns
    -------
    file_path : str
        Absolute file name path of the given file name and directory.

    """

    # Get the system's home directory
    home_dir = os.path.expanduser('~')

    # Join the home directory path with the download path
    compl_dl_path = os.path.join(home_dir, dl_path)

    # Create the download path, if it does not exists already (recursively)
    pathlib.Path(compl_dl_path).mkdir(parents=True, exist_ok=True)

    # Join the home dir. + download dir. with the filename
    file_path = os.path.join(compl_dl_path, filename)

    return file_path


def get_test_file_path(file_path):
    """
    Compute the absolute path to a file within the testing suite.

    Parameters
    ----------
    file_path : str
        Relative filepath of the test file w.r.t. the root directory.

    Returns
    -------
    compl_test_file_path : str
        Absolute filepath to the testing file.

    """

    # Join the root directory of SolarY with the given filepath.
    compl_test_file_path = os.path.join(ROOT_DIR, file_path)

    return compl_test_file_path



"""
test_vec.py

Testing suite for solary/general/vec.py

"""

# Import standard libraries
import math

# Import installed libraries
import pytest

# Import solary
import solary


def test_norm():
    """
    Testing the vector norm computation.

    Returns
    -------
    None.

    """

    # Compute the length of a unit vector
    vec_norm_res1 = solary.general.vec.norm(vector=[1.0, 0.0])
    assert vec_norm_res1 == 1.0

    # Compute the length of a 45 degrees vector
    vec_norm_res2 = solary.general.vec.norm(vector=[1.0, 1.0])
    assert vec_norm_res2 == math.sqrt(2)

    # Example #3
    vec_norm_res3 = solary.general.vec.norm(vector=[5.0, 4.0])
    assert vec_norm_res3 == math.sqrt(41)

    # Example #4
    vec_norm_res4 = solary.general.vec.norm(vector=[1.0, 1.0, 1.0])
    assert vec_norm_res4 == math.sqrt(3)

    # Example #5
    vec_norm_res5 = solary.general.vec.norm(vector=[2.0, 4.0, -5.0, 6.0])
    assert vec_norm_res5 == 9.0


def test_unify():
    """
    Testing the function that normalises a vector.

    Returns
    -------
    None.

    """

    # A unified unit vector must correspond to the input
    unit_vec1 = solary.general.vec.unify(vector=[1.0, 0.0, 0.0])
    assert unit_vec1 == [1.0, 0.0, 0.0]

    # Example #2
    unit_vec2 = solary.general.vec.unify(vector=[5.0, 0.0, 0.0])
    assert unit_vec2 == [1.0, 0.0, 0.0]

    # Example #3
    unit_vec3 = solary.general.vec.unify(vector=[5.0, 5.0, 5.0])
    assert pytest.approx(unit_vec3) == [1.0 / math.sqrt(3), 1.0 / math.sqrt(3), 1.0 / math.sqrt(3)]


def test_dot_prod():
    """
    Testing suite for the computation of the dot product.

    Returns
    -------
    None.

    """

    # Example #1
    dot_res1 = solary.general.vec.dot_prod(vector1=[1.0, 2.0, 3.0], \
                                           vector2=[-2.0, 5.0, 8.0])
    assert dot_res1 == 32.0

    # Example #2
    dot_res2 = solary.general.vec.dot_prod(vector1=[-10.0, 20.0], \
                                           vector2=[-1.0, 0.0])
    assert dot_res2 == 10.0

    # Example #3
    dot_res3 = solary.general.vec.dot_prod(vector1=[23.0, 10.0], \
                                           vector2=[2.0, 0.01])
    assert dot_res3 == 46.1


def test_phase_angle():
    """
    Testing the computation of the phase angle between two vectors.

    Returns
    -------
    None.

    """

    # Example #1: Phase angle of 90 degrees
    angle_res1 = solary.general.vec.phase_angle(vector1=[1.0, 0.0], \
                                                vector2=[0.0, 1.0])
    assert angle_res1 == math.pi / 2.0

    # Example #2: Phase angle of 180 degrees
    angle_res2 = solary.general.vec.phase_angle(vector1=[1.0, 0.0], \
                                                vector2=[-1.0, 0.0])
    assert angle_res2 == math.pi

    # Example #3: Phase angle of 0 degrees
    angle_res3 = solary.general.vec.phase_angle(vector1=[1.0, 0.0], \
                                                vector2=[1.0, 0.0])
    assert angle_res3 == 0.0


def test_substract():
    """
    Testing the substraction of two vectors.

    Returns
    -------
    None.

    """

    # Example #1
    vec_diff1 = solary.general.vec.substract(vector1=[4.0, 7.0], \
                                             vector2=[5.0, 1.0])
    assert vec_diff1 == [-1.0, 6.0]

    # Example #2
    vec_diff2 = solary.general.vec.substract(vector1=[-4.0, -4.0], \
                                             vector2=[-5.0, 9.0])
    assert vec_diff2 == [1.0, -13.0]


def test_inverse():
    """
    Testing the vector inversion.

    Returns
    -------
    None.

    """

    # Example #1
    inverse_vec1 = solary.general.vec.inverse(vector=[1.0, 5.0])
    assert inverse_vec1 == [-1.0, -5.0]

    # Example #2
    inverse_vec2 = solary.general.vec.inverse(vector=[-5.1, -100.0, 0.0])
    assert inverse_vec2 == [5.1, 100.0, 0.0]



import os
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

__project__ = "SolarY"
__author__ = "Dr.-Ing. Thomas Albin"
__version__ = "0.5.6"

from . import _config

from . import tests

from . import auxiliary
from . import general
from . import asteroid
from . import neo

from . import instruments


"""
photometry.py

This module contains functions for photometric purposes.

"""

# Import standard modules
import math

# Import solary
import solary


def appmag2irr(app_mag):
    """
    Convert the apparent magnitude to the corresponding irradiance given in
    W/m^2. The zero point magnitude is provided by the IAU in [1].

    Parameters
    ----------
    app_mag : int or float
        Apparent bolometric magnitude given in mag.

    Returns
    -------
    irradiance : float
        Irradiance given in W/m^2.

    References
    ----------
    [1] https://www.iau.org/static/resolutions/IAU2015_English.pdf

    Examples
    --------
    >>> import solary
    >>> irradiance = solary.general.photometry.appmag2irr(app_mag=8.0)
    >>> irradiance
    1.5887638447672732e-11

    """

    # Load the configuration file that contains the zero point bolometric
    # irradiance
    config = solary.auxiliary.config.get_constants()
    appmag_irr_i0 = float(config['photometry']['appmag_irr_i0'])

    # Convert apparent magnitude to irradiance
    irradiance = 10.0 ** (-0.4 * app_mag + math.log10(appmag_irr_i0))

    return irradiance


def intmag2surmag(intmag, area):
    """
    Convert the integrated magnitude, given over a certain area in the sky to the corresponding
    surface brightness.

    Parameters
    ----------
    intmag : float
        Integrated magnitude given in mag.
    area : float
        Area of the object given in arcsec^2.

    Returns
    -------
    surface_mag : float
        Surface brightness of the object given in mag/arcsec^2.

    """

    # Compute the surface brightness
    surface_mag = intmag + 2.5 * math.log10(area)

    return surface_mag


def surmag2intmag(surmag, area):
    """
    Convert the surface brightness and a sky area to an integrated magnitude. The integrated
    magnitude can later be used to e.g., convert the night sky background brightness to an
    irradiance.

    Parameters
    ----------
    surmag : float
        Surface brightness given in mag/arcsec^2.
    area : float
        Area given in arcsec^2.

    Returns
    -------
    intmag : float
        Integrated magnitude given in mag.

    """

    # Compute the integrated magnitude
    intmag = surmag - 2.5 * math.log10(area)

    return intmag


def phase_func(index, phase_angle):
    """
    Phase function that is needed for the H-G visual / apparent magnitude function. The function
    has two versions, depending on the index ('1' or '2'). See [1].

    Parameters
    ----------
    index : str
        Phase function index / version. '1' or '2'.
    phase_angle : float
        Phase angle of the asteroid in radians (Angle as seen from the asteroid, pointing to
                                                a light source (Sun) and the observer (Earth)).

    Returns
    -------
    phi : float
        Phase function result.

    See Also
    --------
    hg_app_mag : Computing the visual / apparent magnitude of an object

    References
    ----------
    [1] https://www.britastro.org/asteroids/dymock4.pdf

    Examples
    --------
    >>> import math
    >>> import solary
    >>> phi1 = solary.general.photometry.phase_func(index=1, phase_angle=math.pi/4.0)
    >>> phi1
    0.14790968630394927
    >>> phi2 = solary.general.photometry.phase_func(index=2, phase_angle=math.pi/4.0)
    >>> phi2
    0.5283212147726485

    """

    # Dictionaries that contain the A and B constants, depending on the index version
    a_factor = {1: 3.33, \
                2: 1.87}
    b_factor = {1: 0.63, \
                2: 1.22}

    # Phase function
    phi = math.exp(-1.0 * a_factor[index] * ((math.tan(0.5 * phase_angle) ** b_factor[index])))

    # Return the phase function result
    return phi


def reduc_mag(abs_mag, phase_angle, slope_g=0.15):
    """
    Function to compute the reduced magnitude of an object. This function is needed for the
    H-G visual / apparent magnitude function. See [1]

    Parameters
    ----------
    abs_mag : float
        Absolute magnitude of the object.
    phase_angle : float
        Phase angle of the object w.r.t. the illumination source and observer.
    slope_g : float, optional
        Slope parameter G for the reduced magnitude. The set default value can be applied for
        asteroids with unknown slope parameter and the interval is (0, 1). The default is 0.15.

    Returns
    -------
    reduced_magnitude : float
        Reduced magnitude of the object.

    See Also
    --------
    hg_app_mag : Computing the visual / apparent magnitude of an object

    References
    ----------
    [1] https://www.britastro.org/asteroids/dymock4.pdf

    Examples
    --------
    >>> import math
    >>> import solary
    >>> reduced_magnitude = solary.general.photometry.reduc_mag(abs_mag=10.0, \
                                                                phase_angle=math.pi/4.0, \
                                                                slope_g=0.10)
    >>> reduced_magnitude
    11.826504643588578

    Per default, the slope parameter G is set to 0.15 and fits well for most asteroids

    >>> reduced_magnitude = solary.general.photometry.reduc_mag(abs_mag=10.0, \
                                                                phase_angle=math.pi/4.0)
    >>> reduced_magnitude
    11.720766748872016

    """

    # Compute the reduced magnitude based on the equations given in the references [1]
    reduced_magnitude = abs_mag - 2.5 * math.log10((1.0 - slope_g) \
                                                   * phase_func(index=1, phase_angle=phase_angle) \
                                                   + slope_g \
                                                   * phase_func(index=2, phase_angle=phase_angle))

    return reduced_magnitude


def hg_app_mag(abs_mag, vec_obj2obs, vec_obj2ill, slope_g=0.15):
    """
    Compute the visual / apparent magnitude of an asteroid, based on the H-G system [1], where H
    represents the absolute magnitude and G represents the magnitude slope parameter.

    Parameters
    ----------
    abs_mag : float
        Absolute magnitude.
    vec_obj2obs : list
        3 dimensional vector the contains the directional information (x, y, z) from the asteroid
        to the observer given in AU.
    vec_obj2ill : list
        3 dimensional vector the contains the directional information (x, y, z) from the asteroid
        to the illumination source given in AU.
    slope_g : float, optional
        Slope parameter G for the reduced magnitude. The set default value can be applied for
        asteroids with unknown slope parameter and the interval is (0, 1). The default is 0.15.

    Returns
    -------
    app_mag : float
        Apparent / visual (bolometric) magnitude of the asteroid as seen from the observer.

    References
    ----------
    [1] https://www.britastro.org/asteroids/dymock4.pdf

    Examples
    --------
    >>> import solary
    >>> apparent_magnitude = solary.general.photometry.hg_app_mag(abs_mag=10.0, \
                                                                  vec_obj2obs=[-1.0, 0.0, 0.0], \
                                                                  vec_obj2ill=[-2.0, 0.0, 0.0], \
                                                                  slope_g=0.10)
    >>> apparent_magnitude
    11.505149978319906

    """

    # Compute the length of the two input vectors
    vec_obj2obs_norm = solary.general.vec.norm(vec_obj2obs)
    vec_obj2ill_norm = solary.general.vec.norm(vec_obj2ill)

    # Compute the phase angle of the asteroid
    obj_phase_angle = solary.general.vec.phase_angle(vec_obj2obs, vec_obj2ill)

    # Compute the reduced magnitude of the asteroid
    red_mag = reduc_mag(abs_mag, obj_phase_angle, slope_g)

    # Merge all information and compute the apparent magnitude of the asteroid as seen from the
    # observer
    app_mag = red_mag + 5.0 * math.log10(vec_obj2obs_norm * vec_obj2ill_norm)

    return app_mag



"""
test_parse.py

Testing suite for solary/auxiliary/parse.py

"""

# Import standard libraries
import os

# Import solary
import solary


def test_comp_md5():
    """
    Test function to check if the function comp_md5 computes the correct MD5 hash value of a mock
    up file.

    Returns
    -------
    None.

    """

    # Create mockup file
    mockup_file = 'mockup.txt'
    open(mockup_file, 'wb').close()

    # Compute the MD5 has and compare the result with the expectation
    md5_mockup = solary.auxiliary.parse.comp_md5(mockup_file)
    assert md5_mockup == 'd41d8cd98f00b204e9800998ecf8427e'

    # Remove the mockup file from the system
    os.remove(mockup_file)
